/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package vbsearch;

import java.util.*;
import vbsearch.util.*;

/**
 *  Description of the Class
 *
 * @author     Samppa Saarela
 * @created    26. elokuuta 2003
 */
public interface Category extends VBSResource {
	/**
	 *  Gets the categoryID attribute of the Category object
	 *
	 * @return    The categoryID value
	 */
	public PrefixLabel getCategoryID();


	/**
	 *  Gets the children attribute of the Category object
	 *
	 * @param  direct  Description of the Parameter
	 * @return         The children value
	 */
	public Iterator getSubCategories(boolean direct);


	/**
	 *  Gets the parent attribute of the Category object
	 *
	 * @return    The parent value
	 */
	public Category getParent();


	/**
	 *  Gets the label attribute of the Category object
	 *
	 * @param  lang  Description of the Parameter
	 * @return       The label value
	 */
	public String getLabel(String lang);


	/**
	 *  Gets the bookmarks attribute of the Category object
	 *
	 * @param  direct  Description of the Parameter
	 * @return         The bookmarks value
	 */
	public Iterator getBookmarks(boolean direct);


	/**
	 *  Gets the document attribute of the Category object
	 *
	 * @return    The document value
	 */
	public String getDocument();
}

