/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package vbsearch;

import com.hp.hpl.jena.ontology.OntModelSpec;
import java.io.*;
import java.io.IOException;
import java.util.*;
import seco.util.conf.*;
import vbsearch.util.*;

/**
 *  Description of the Class
 *
 *@author     Samppa
 *@created    27. elokuuta 2003
 */
public class VBSConf {
    /**
     *  Description of the Field
     */
    public static String BASE_URI;
    /**
     *  Description of the Field
     */
    public static String BM_NS;

    /**
     *  Description of the Field
     */
    public static String[] LITERAL_BOOKMARK_INDEX_PROPERTIES;

    /**
     *  Description of the Field
     */
    public static String[] LITERAL_CATEGORY_INDEX_PROPERTIES;
    /**
     *  Description of the Field
     */
    public static String OGT_NS;

    /**
     *  Description of the Field
     */
    public static OntModelSpec ONT_MODEL_SPEC;
    /**
     *  Description of the Field
     */
    public static String OWL_NS;
    /**
     *  Description of the Field
     */
    public static String RDFS_NS;
    /**
     *  Description of the Field
     */
    public static String RDF_NS;

    static {
        Constants constants = new Constants();
        Properties cnf = new Properties();
        try {
            cnf.load(VBSearch.class.getResource("/vbsearch.conf").openStream());
            constants.putConstants(cnf);
        } catch (IOException ioe) {
            ioe.printStackTrace();
            System.exit(0);
        }
        if (constants.containsKey("uri.xmlns.rdf")) {
            RDF_NS = constants.getString("uri.xmlns.rdf");
        }
        if (constants.containsKey("uri.xmlns.rdfs")) {
            RDFS_NS = constants.getString("uri.xmlns.rdfs");
        }
        if (constants.containsKey("uri.xmlns.ogt")) {
            OGT_NS = constants.getString("uri.xmlns.ogt");
        }
        if (constants.containsKey("uri.xmlns.bm")) {
            BM_NS = constants.getString("uri.xmlns.bm");
        }
        if (constants.containsKey("uri.xmlns.owl")) {
            OWL_NS = constants.getString("uri.xmlns.owl");
        }
        if (constants.containsKey("uri.base")) {
            BASE_URI = constants.getString("uri.base");
        }

        LITERAL_BOOKMARK_INDEX_PROPERTIES = constants.containsKey("literalBookmarkIndexProperties")
                 ? constants.getArray("literalBookmarkIndexProperties")
                 : new String[]{};
        LITERAL_CATEGORY_INDEX_PROPERTIES = constants.containsKey("literalCategoryIndexProperties")
                 ? constants.getArray("literalCategoryIndexProperties")
                 : new String[]{};

        String ontModelSpecField = constants.getString("jena.ontModelSpec");
        try {
            Class ontModelSpec = OntModelSpec.class;
            ONT_MODEL_SPEC = (OntModelSpec) ontModelSpec.getField(ontModelSpecField).get(null);
        } catch (Exception e) {
            System.err.println("CONFIGURATION ERROR: Unable to load jena.ontModelSpec (" + ontModelSpecField + ")");
            System.exit(0);
        }
    }
}

