/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package vbsearch;

import java.net.URISyntaxException;
import java.util.*;

/**
 *  Description of the Class
 *
 *@author     Samppa
 *@created    26. elokuuta 2003
 */
public interface VBSIndex {
    /**
     *  Adds a feature to the Category attribute of the VBSIndex object
     *
     *@param  catinfo                       The feature to be added to the
     *      Category attribute
     *@return                               Description of the Return Value
     *@exception  UnknownResourceException  Description of the Exception
     *@exception  URISyntaxException        Description of the Exception
     *@exception  NullPointerException      Description of the Exception
     */
    public Category addCategory(Category catinfo) throws UnknownResourceException, URISyntaxException, NullPointerException;


    /**
     *  Adds a feature to the Bookmark attribute of the VBSIndex object
     *
     *@param  bookmark                      The feature to be added to the
     *      Bookmark attribute
     *@return                               Description of the Return Value
     *@exception  UnknownResourceException  Description of the Exception
     *@exception  URISyntaxException        Description of the Exception
     *@exception  NullPointerException      Description of the Exception
     */
    public Bookmark addBookmark(Bookmark bookmark) throws UnknownResourceException, URISyntaxException, NullPointerException;


    /**
     *  Gets the category attribute of the VBSIndex object
     *
     *@param  catid  Description of the Parameter
     *@return        The category value
     */
    public Category getCategory(String catid);


    /**
     *  Description of the Method
     *
     *@param  category                      Description of the Parameter
     *@exception  UnknownResourceException  Description of the Exception
     *@exception  NullPointerException      Description of the Exception
     */
    public void removeCategory(Category category) throws UnknownResourceException, NullPointerException;


    /**
     *  Description of the Method
     *
     *@param  bookmark                      Description of the Parameter
     *@exception  UnknownResourceException  Description of the Exception
     *@exception  NullPointerException      Description of the Exception
     */
    public void removeBookmark(Bookmark bookmark) throws UnknownResourceException, NullPointerException;


    /**
     *  Gets the allBookmarks attribute of the VBSIndex object
     *
     *@return    The allBookmarks value
     */
    public Collection getAllBookmarks();


    /**
     *  Gets the bookmark attribute of the VBSIndex object
     *
     *@param  catid  Description of the Parameter
     *@return        The bookmark value
     */
    public Bookmark getBookmark(String catid);


    /**
     *@param  uri
     *@return
     */
    public Bookmark getBookmarkByURI(String uri);


    /**
     *  Gets the categoriesByKeyword attribute of the VBSIndex object
     *
     *@param  categories  Description of the Parameter
     *@param  keyword     Description of the Parameter
     *@return             The categoriesByKeyword value
     */
    public Iterator getCategoriesByKeyword(String keyword, Collection categories);


    /**
     *  Gets the bookmarksByKeyword attribute of the VBSIndex object
     *
     *@param  keyword  Description of the Parameter
     *@return          The bookmarksByKeyword value
     */
    public Iterator getBookmarksByKeyword(String keyword);


    /**
     *  Gets the bookmarkCount attribute of the VBSIndex object
     *
     *@return    The bookmarkCount value
     */
    public int getBookmarkCount();


    /**
     *  Gets the categoryBase attribute of the VBSIndex object
     *
     *@return    The categoryBase value
     */
    public VBSResource getCategoryBase();


    /**
     *  Gets the categoriesByProjection attribute of the VBSIndex object
     *
     *@param  projectedFrom  Description of the Parameter
     *@param  categories     Description of the Parameter
     *@return                The categoriesByProjection value
     */
    public Iterator getCategoriesByProjection(String projectedFrom, Collection categories);


    /**
     *  Gets the roots attribute of the VBSIndex object
     *
     *@return    The roots value
     */
    public Iterator getRoots();


    /**
     *  Sets the categoryBase attribute of the VBSIndex object
     *
     *@param  categoryBase  The new categoryBase value
     */
    public void setCategoryBase(VBSResource categoryBase);
}

