/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package vbsearch;

import java.io.Serializable;
import java.net.*;

/**
 *  Description of the Interface
 *
 * @author     Samppa
 * @created    26. elokuuta 2003
 */
public class VBSProperty implements Serializable {
	/**
	 *  Description of the Field
	 */
	public final ValueType ANON      = new ValueType();

	/**
	 *  Description of the Field
	 */
	public final ValueType LITERAL   = new ValueType();
	/**
	 *  Description of the Field
	 */
	public final ValueType RESOURCE  = new ValueType();

	/**
	 *  Description of the Field
	 */
	private String id;
	/**
	 *  Description of the Field
	 */
	private String lang;
	/**
	 *  Description of the Field
	 */
	private ValueType type;
	/**
	 *  Description of the Field
	 */
	private String value;


	/**
	 *  Constructor for the VBSProperty object
	 *
	 * @param  id                        Description of the Parameter
	 * @param  value                     Description of the Parameter
	 * @param  lang                      Description of the Parameter
	 * @param  type                      Description of the Parameter
	 * @exception  NullPointerException  Description of the Exception
	 * @exception  URISyntaxException    Description of the Exception
	 */
	public VBSProperty(String id, String value, ValueType type, String lang) throws NullPointerException, URISyntaxException {
		if (id == null || value == null || type == null) {
			throw new NullPointerException();
		} else {
			if (type == RESOURCE) {
				new URI(value);
			}

			this.id = id;
			this.value = value;
			this.type = type;
			this.lang = lang;
		}
	}


	/**
	 *  Gets the iD attribute of the VBSProperty object
	 *
	 * @return    The iD value
	 */
	public String getID() {
		return this.id;
	}


	/**
	 *  Gets the language attribute of the VBSProperty object
	 *
	 * @return    The language value
	 */
	public String getLanguage() {
		return this.lang;
	}


	/**
	 *  Gets the value attribute of the VBSProperty object
	 *
	 * @return    The value value
	 */
	public String getValue() {
		return this.value;
	}


	/**
	 *  Gets the type attribute of the VBSProperty object
	 *
	 * @return    The type value
	 */
	public ValueType getValueType() {
		return this.type;
	}


	/**
	 *  Description of the Method
	 *
	 * @param  lang  Description of the Parameter
	 * @return       Description of the Return Value
	 */
	public boolean matchLanguage(String lang) {
		boolean rv  = false;
		if (lang == null) {
			rv = true;
		} else if (this.lang != null && lang.equals(this.lang)) {
			rv = true;
		}
		return rv;
	}


	/**
	 *  Description of the Class
	 *
	 * @author     Samppa
	 * @created    26. elokuuta 2003
	 */
	public class ValueType implements Serializable {
		/**
		 *  Constructor for the ValueType object
		 */
		private ValueType() { }
	}
}

