/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package vbsearch;

import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.util.iterator.*;
import java.io.*;

/**
 *  Description of the Class
 *
 *@author     Samppa
 *@created    September 12, 2003
 */
public class VBSVocabulary {
    /**
     *  Description of the Field
     */
    public static Property ANY_PROPERTYP;
    /**
     *  Description of the Field
     */
    public static Property acceptLanguageP;
    /**
     *  Description of the Field
     */
    public static OntClass bookmarkC;
    /**
     *  Description of the Field
     */
    public static Property bookmarkKeywordP;
    /**
     */
    public static Property bookmarkP;
    /**
     *  Description of the Field
     */
    public static OntClass bookmarkSelectorC;
    /**
     *  Description of the Field
     */
    public static OntClass categoryBaseC;
    /**
     *  Description of the Field
     */
    public static OntClass categoryC;
    /**
     *  Description of the Field
     */
    public static OntClass categoryConstraintC;
    /**
     *  Description of the Field
     */
    public static Property categoryKeywordP;

    /**
     *  Description of the Field
     */
    public static Property categoryP;

    /**
     *  Description of the Field
     */
    public static Property catidP;
    /**
     *  Description of the Field
     */
    public static Property constraintP;
    /**
     *  Description of the Field
     */
    public static OntClass facetSelectorC;
    /**
     *  Description of the Field
     */
    public static Property filterP;
    /**
     *  Description of the Field
     */
    public static Property hasTopicP;

    /**
     *  Description of the Field
     */
    public static Property incBookmarkHitsP;
    /**
     *  Description of the Field
     */
    public static Property incBookmarkPropertyP;
    /**
     *  Description of the Field
     */
    public static Property incBookmarksP;

    /**
     *  Description of the Field
     */
    public static Property incCategoriesP;
    /**
     *  Description of the Field
     */
    public static Property incCategoryPropertyP;
    /**
     *  Description of the Field
     */
    public static Property incDirectBookmarkHitsP;
    /**
     *  Description of the Field
     */
    public static Property incPathBookmarkHitsP;
    /**
     *  Description of the Field
     */
    public static Property incPathP;
    /**
     *  Description of the Field
     */
    public static Property incPathSubCategoryHitsP;
    /**
     *  Description of the Field
     */
    public static Property incPositionP;
    /**
     *  Description of the Field
     */
    public static Property incRemainderP;
    /**
     *  Description of the Field
     */
    public static Property incSubCategoriesP;
    /**
     */
    public static Property incSubCategoryHitsP;
    /**
     *  Description of the Field
     */
    public static Property incTopicsP;
    /**
     *  Description of the Field
     */
    public static Property keywordP;
    /**
     *  Description of the Field
     */
    public static Property limitP;
    /**
     */
    public static Property maxBookmarksP;
    /**
     */
    public static Property maxSubCategoriesP;
    /**
     *  Description of the Field
     */
    private static OntModel model;
    /**
     *  Description of the Field
     */
    public static Property onCategoryP;
    /**
     *  Description of the Field
     */
    public static Property projectedFromP;
    /**
     *  Description of the Field
     */
    public static Property projectionOfP;
    /**
     *  Description of the Field
     */
    public static Property recallsP;
    /**
     *  Description of the Field
     */
    public static OntResource rootCategoryR;
    /**
     */
    public static Property skipBookmarksP;
    /**
     */
    public static Property skipSubCategoriesP;
    /**
     *  Description of the Field
     */
    public static Property subCategoriesP;
    /**
     *  Description of the Field
     */
    public static Property subCategoryOfP;
    /**
     *  Description of the Field
     */
    public static Property topicOfP;
    /**
     *  Description of the Field
     */
    public static Property versionInfoP;
    static {
        try {
            model = (OntModel) ModelFactory.createOntologyModel(VBSConf.ONT_MODEL_SPEC, null)
                    .read(VBSVocabulary.class.getResource("/ontogator.owl").toString())
                    .read(VBSVocabulary.class.getResource("/bookmark.rdf").toString());

            // CategoryOntology properties
            projectionOfP = model.getProperty(VBSConf.OGT_NS + "projectionOf");
            subCategoryOfP = model.getProperty(VBSConf.OGT_NS + "subCategoryOf");
            recallsP = model.getProperty(VBSConf.BM_NS + "recalls");
            topicOfP = model.getProperty(VBSConf.OGT_NS + "topicOf");
            hasTopicP = model.getProperty(VBSConf.OGT_NS + "hasTopic");
            catidP = model.getProperty(VBSConf.OGT_NS + "catid");
            subCategoriesP = model.getProperty(VBSConf.OGT_NS + "subCategories");
            versionInfoP = model.getProperty(VBSConf.OWL_NS + "versionInfo");

            // CategoryOntology resources
            rootCategoryR = (OntResource) model.getResource(VBSConf.OGT_NS + "RootCategory").as(OntResource.class);

            // CategoryOntology classes
            categoryC = model.getOntClass(VBSConf.OGT_NS + "Category");
            bookmarkC = model.getOntClass(VBSConf.BM_NS + "Bookmark");
            categoryBaseC = model.getOntClass(VBSConf.OGT_NS + "CategoryBase");

            // QueryOntology properties
            incBookmarksP = model.getProperty(VBSConf.OGT_NS + "incBookmarks");
            incCategoriesP = model.getProperty(VBSConf.OGT_NS + "incCategories");
            incPathP = model.getProperty(VBSConf.OGT_NS + "incPath");
            incSubCategoriesP = model.getProperty(VBSConf.OGT_NS + "incSubCategories");
            incTopicsP = model.getProperty(VBSConf.OGT_NS + "incTopics");
            categoryP = model.getProperty(VBSConf.OGT_NS + "category");
            projectedFromP = model.getProperty(VBSConf.OGT_NS + "projectedFrom");
            bookmarkKeywordP = model.getProperty(VBSConf.OGT_NS + "bookmarkKeyword");
            categoryKeywordP = model.getProperty(VBSConf.OGT_NS + "categoryKeyword");
            constraintP = model.getProperty(VBSConf.OGT_NS + "constraint");
            acceptLanguageP = model.getProperty(VBSConf.OGT_NS + "acceptLanguage");
            incBookmarkPropertyP = model.getProperty(VBSConf.OGT_NS + "incBookmarkProperty");
            incCategoryPropertyP = model.getProperty(VBSConf.OGT_NS + "incCategoryProperty");
            filterP = model.getProperty(VBSConf.OGT_NS + "filter");
            bookmarkP = model.getProperty(VBSConf.OGT_NS + "bookmark");
            incBookmarkHitsP = model.getProperty(VBSConf.OGT_NS + "incBookmarkHits");
            incPathBookmarkHitsP = model.getProperty(VBSConf.OGT_NS + "incPathBookmarkHits");
            incDirectBookmarkHitsP = model.getProperty(VBSConf.OGT_NS + "incDirectBookmarkHits");
            incRemainderP = model.getProperty(VBSConf.OGT_NS + "incRemainder");
            maxBookmarksP = model.getProperty(VBSConf.OGT_NS + "maxBookmarks");
            maxSubCategoriesP = model.getProperty(VBSConf.OGT_NS + "maxSubCategories");
            incSubCategoryHitsP = model.getProperty(VBSConf.OGT_NS + "incSubCategoryHits");
            incPathSubCategoryHitsP = model.getProperty(VBSConf.OGT_NS + "incPathSubCategoryHits");
            skipBookmarksP = model.getProperty(VBSConf.OGT_NS + "skipBookmarks");
            skipSubCategoriesP = model.getProperty(VBSConf.OGT_NS + "skipSubCategories");
            incPositionP = model.getProperty(VBSConf.OGT_NS + "incPosition");
            onCategoryP = model.getProperty(VBSConf.OGT_NS + "onCategory");
            limitP = model.getProperty(VBSConf.OGT_NS + "limit");
            ANY_PROPERTYP = model.getProperty(VBSConf.OGT_NS + "ANY_PROPERTY");

            // QueryOntology classes
            facetSelectorC = model.getOntClass(VBSConf.OGT_NS + "FacetSelector");
            bookmarkSelectorC = model.getOntClass(VBSConf.OGT_NS + "BookmarkSelector");
            categoryConstraintC = model.getOntClass(VBSConf.OGT_NS + "CategoryConstraint");
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }
    }


    /**
     *  Gets the model attribute of the VBSVocabulary object
     *
     *@return    The model value
     */
    public static OntModel getModel() {
        return model;
    }
}

