/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package vbsearch.impl.jena;

import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.util.iterator.*;
import java.util.*;
import seco.util.iterator.*;
import vbsearch.*;
import vbsearch.impl.*;
import vbsearch.util.PrefixLabel;

/**
 *  Description of the Class
 *
 * @author     Samppa
 * @created    September 11, 2003
 */
public class JenaCategory extends JenaResource implements Category {
	/**
	 *  Description of the Field
	 */
	private Category parent;


	/**
	 *  Constructor for the JenaCategory object
	 *
	 * @param  category  Description of the Parameter
	 * @param  parent    Description of the Parameter
	 * @param  document  Description of the Parameter
	 */
	public JenaCategory(Category parent, OntResource category, String document) {
		super(category, document);
		this.parent = parent;
	}


	/**
	 *  Gets the bookmarks attribute of the JenaCategory object
	 *
	 * @param  direct  Description of the Parameter
	 * @return         The bookmarks value
	 */
	public Iterator getBookmarks(boolean direct) {
		Collection rv    = new LinkedList();
		BasicIterator i  = listPropertyValues(getResource(), VBSVocabulary.topicOfP, OntResource.class);
		if (i != null) {
			while (i.hasNext()) {
				Bookmark bm  = new JenaBookmark((OntResource)i.next(), getDocument());
				rv.add(bm);
				if (!direct) {
					for (Iterator j = getSubCategories(true); j.hasNext(); ) {
						for (Iterator bmiter = ((Category)j.next()).getBookmarks(true); bmiter.hasNext(); ) {
							rv.add(bmiter.next());
						}
					}
				}
			}
		}
		//return new BasicIterator(rv.iterator(), new UniqueFilter());
		return rv.iterator();
	}


	/**
	 *  Gets the categoryID attribute of the JenaCategory object
	 *
	 * @return    The categoryID value
	 */
	public PrefixLabel getCategoryID() {
		return null;
	}


	/**
	 *  Gets the parent attribute of the JenaCategory object
	 *
	 * @return    The parent value
	 */
	public Category getParent() {
		return this.parent;
	}


	/**
	 *  Gets the subCategories attribute of the JenaCategory object
	 *
	 * @param  direct  Description of the Parameter
	 * @return         The subCategories value
	 */
	public Iterator getSubCategories(boolean direct) {
		Collection rv    = new LinkedList();
		BasicIterator i  = listPropertyValues(getResource(), VBSVocabulary.subCategoriesP, OntResource.class);
		if (i != null) {
			while (i.hasNext()) {
				Category subCategory  = new JenaCategory(this, (OntResource)i.next(), getDocument());
				rv.add(subCategory);
				if (!direct) {
					for (Iterator sciter = subCategory.getSubCategories(direct); sciter.hasNext(); ) {
						rv.add(sciter.next());
					}
				}
			}
		}
		//return new BasicIterator(rv.iterator(), new UniqueFilter());
		return rv.iterator();
	}
}


