/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package vbsearch.impl.jena;

import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.util.iterator.*;
import java.util.*;
import seco.util.iterator.*;
import vbsearch.*;
import vbsearch.impl.*;
import vbsearch.query.*;
import vbsearch.util.*;

/**
 *  Description of the Class
 *
 *@author     Samppa
 *@created    September 12, 2003
 */
public class JenaCategoryConstraint extends JenaResource implements CategoryConstraint {

    /**
     *  Constructor for the CategoryConstraint object
     *
     *@param  resource  Description of the Parameter
     */
    public JenaCategoryConstraint(OntResource resource) {
        super(resource);
//		this.resource = resource;
    }


    /**
     *@return
     */
    public Iterator getBookmark() {
        return new BasicIterator(getResource().listPropertyValues(VBSVocabulary.bookmarkP),
            new IterationMap() {
                public Object map(Object o) {
                    return o.toString();
                }
            });
    }


    /**
     *  Gets the keyword attribute of the CategoryConstraint object
     *
     *@return    The keyword value
     */
    public Iterator getBookmarkKeyword() {
        return listPropertyValues(getResource(), VBSVocabulary.bookmarkKeywordP, Literal.class);
    }


    /**
     *  Gets the category attribute of the FacetSelector object
     *
     *@return    The category value
     */
    public Iterator getCategory() {
        return listPropertyValues(getResource(), VBSVocabulary.categoryP, Literal.class);
    }


    /**
     *  Gets the keyword attribute of the CategoryConstraint object
     *
     *@return    The keyword value
     */
    public Iterator getCategoryKeyword() {
        return listPropertyValues(getResource(), VBSVocabulary.categoryKeywordP, Literal.class);
    }



    /**
     *  Gets the projectedFrom attribute of the FacetSelector object
     *
     *@return    The projectedFrom value
     */
    public Iterator getProjectedFrom() {
        return listPropertyValues(getResource(), VBSVocabulary.projectedFromP, OntResource.class);
    }
}

