/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package vbsearch.impl.jena;

import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.util.iterator.*;
import java.util.*;
import seco.util.iterator.*;
import vbsearch.*;
import vbsearch.impl.*;
import vbsearch.query.*;
import vbsearch.util.*;

/**
 *  Description of the Class
 *
 *@author     Samppa
 *@created    September 12, 2003
 */
public class JenaFacetSelector extends JenaCategoryConstraint implements FacetSelector {

    private String intersectionKey = null;


    /**
     *  Constructor for the FacetSelector object
     *
     *@param  facetSelector  Description of the Parameter
     */
    public JenaFacetSelector(OntResource facetSelector) {
        super(facetSelector);
    }


    /**
     *  Gets the acceptLanguage attribute of the JenaFacetSelector object
     *
     *@return    The acceptLanguage value
     */
    public String getAcceptLanguage() {
        String rv = null;
        // default
        if (getResource().hasProperty(VBSVocabulary.acceptLanguageP)) {
            rv = getResource().getPropertyValue(VBSVocabulary.acceptLanguageP).toString();
        }
        return rv;
    }


    /**
     *  Gets the constraints attribute of the VBSSelector object
     *
     *@return    The constraints value
     */
    public Iterator getConstraints() {
        return new BasicIterator(JenaResource.listPropertyValues(getResource(), VBSVocabulary.constraintP, OntResource.class),
            new IterationMap() {
                public Object map(Object o) {
                    return new JenaCategoryConstraint((OntResource) o);
                }
            });
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public int getFilter() {
        int rv = POST_QUERY;
        // default
        if (getResource().hasProperty(VBSVocabulary.filterP)) {
            String value = getResource().getPropertyValue(VBSVocabulary.filterP).toString();
            if (value.equals("PRE-QUERY")) {
                rv = PRE_QUERY;
            } else if (value.equals("NONE")) {
                rv = NONE;
            }
        }
        return rv;
    }


    /**
     *  Gets the incHits attribute of the JenaFacetSelector object
     *
     *@return    The incHits value
     */
    public boolean getIncBookmarkHits() {
        boolean rv = true;
        // default
        if (getResource().hasProperty(VBSVocabulary.incBookmarkHitsP)) {
            rv = new Boolean(getResource().getPropertyValue(VBSVocabulary.incBookmarkHitsP).toString())
                    .booleanValue();
        }
        return rv;
    }


    /**
     *  Gets the incBookmarkProperty attribute of the JenaVBSSelector object
     *
     *@return    The incBookmarkProperty value
     */
    public Iterator getIncBookmarkProperty() {
        if (getResource().hasProperty(VBSVocabulary.incBookmarkPropertyP, VBSVocabulary.ANY_PROPERTYP)) {
            return null;
        } else {
            return new BasicIterator(getResource().listPropertyValues(VBSVocabulary.incBookmarkPropertyP),
                new IterationMap() {
                    public Object map(Object o) {
                        return o.toString();
                    }
                });
        }
    }


    /**
     *  Gets the incBookmarks attribute of the FacetSelector object
     *
     *@return    The incBookmarks value
     */
    public boolean getIncBookmarks() {
        boolean rv = false;
        // default
        if (getResource().hasProperty(VBSVocabulary.incBookmarksP)) {
            rv = new Boolean(getResource().getPropertyValue(VBSVocabulary.incBookmarksP).toString())
                    .booleanValue();
        }
        return rv;
    }


    /**
     *  Gets the incCategories attribute of the JenaFacetSelector object
     *
     *@return    The incCategories value
     */
    public boolean getIncCategories() {
        boolean rv = true;
        // default
        if (getResource().hasProperty(VBSVocabulary.incCategoriesP)) {
            rv = new Boolean(getResource().getPropertyValue(VBSVocabulary.incCategoriesP).toString())
                    .booleanValue();
        }
        return rv;
    }


    /**
     *  Gets the incProperty attribute of the JenaFacetSelector object
     *
     *@return    The incProperty value
     */
    public Iterator getIncCategoryProperty() {
        if (getResource().hasProperty(VBSVocabulary.incCategoryPropertyP, VBSVocabulary.ANY_PROPERTYP)) {
            return null;
        } else {
            return new BasicIterator(getResource().listPropertyValues(VBSVocabulary.incCategoryPropertyP),
                new IterationMap() {
                    public Object map(Object o) {
                        return o.toString();
                    }
                });
        }
    }


    /**
     *  Gets the incDirectBookmarkHits attribute of the JenaFacetSelector object
     *
     *@return    The incDirectBookmarkHits value
     */
    public boolean getIncDirectBookmarkHits() {
        boolean rv = false;
        // default
        if (getResource().hasProperty(VBSVocabulary.incDirectBookmarkHitsP)) {
            rv = new Boolean(getResource().getPropertyValue(VBSVocabulary.incDirectBookmarkHitsP).toString())
                    .booleanValue();
        }
        return rv;
    }


    /**
     *  Gets the incPath attribute of the FacetSelector object
     *
     *@return    The incPath value
     */
    public boolean getIncPath() {
        boolean rv = true;
        // default
        if (getResource().hasProperty(VBSVocabulary.incPathP)) {
            rv = new Boolean(getResource().getPropertyValue(VBSVocabulary.incPathP).toString())
                    .booleanValue();
        }
        return rv;
    }


    /**
     *  Gets the incHits attribute of the JenaFacetSelector object
     *
     *@return    The incHits value
     */
    public boolean getIncPathBookmarkHits() {
        boolean rv = false;
        // default
        if (getResource().hasProperty(VBSVocabulary.incPathBookmarkHitsP)) {
            rv = new Boolean(getResource().getPropertyValue(VBSVocabulary.incPathBookmarkHitsP).toString())
                    .booleanValue();
        }
        return rv;
    }


    /**
     *  Gets the incPathSubCategoryHits attribute of the JenaFacetSelector
     *  object
     *
     *@return    The incPathSubCategoryHits value
     */
    public boolean getIncPathSubCategoryHits() {
        boolean rv = false;
        // default
        if (getResource().hasProperty(VBSVocabulary.incPathSubCategoryHitsP)) {
            rv = new Boolean(getResource().getPropertyValue(VBSVocabulary.incPathSubCategoryHitsP).toString())
                    .booleanValue();
        }
        return rv;
    }


    /**
     *  Gets the incPosition attribute of the JenaFacetSelector object
     *
     *@return    The incPosition value
     */
    public boolean getIncPosition() {
        boolean rv = false;
        // default
        if (getResource().hasProperty(VBSVocabulary.incPositionP)) {
            rv = new Boolean(getResource().getPropertyValue(VBSVocabulary.incPositionP).toString())
                    .booleanValue();
        }
        return rv;
    }


    /**
     *  Gets the incRemainder attribute of the JenaFacetSelector object
     *
     *@return    The incRemainder value
     */
    public boolean getIncRemainder() {
        boolean rv = false;
        // default
        if (getResource().hasProperty(VBSVocabulary.incRemainderP)) {
            rv = new Boolean(getResource().getPropertyValue(VBSVocabulary.incRemainderP).toString())
                    .booleanValue();
        }
        return rv;
    }


    /**
     *  Gets the incSubCategories attribute of the FacetSelector object
     *
     *@return    The incSubCategories value
     */
    public int getIncSubCategories() {
        int rv = NONE;
        // default
        if (getResource().hasProperty(VBSVocabulary.incSubCategoriesP)) {
            String value = getResource().getPropertyValue(VBSVocabulary.incSubCategoriesP).toString();
            if (value.equals("ALL")) {
                rv = ALL;
            } else if (value.equals("DIRECT")) {
                rv = DIRECT;
            }
        }
        return rv;
    }


    /**
     *  Gets the incSubCategoryHits attribute of the JenaFacetSelector object
     *
     *@return    The incSubCategoryHits value
     */
    public boolean getIncSubCategoryHits() {
        boolean rv = false;
        // default
        if (getResource().hasProperty(VBSVocabulary.incSubCategoryHitsP)) {
            rv = new Boolean(getResource().getPropertyValue(VBSVocabulary.incSubCategoryHitsP).toString())
                    .booleanValue();
        }
        return rv;
    }


    /**
     *  Gets the incTopics attribute of the FacetSelector object
     *
     *@return    The incTopics value
     */
    public int getIncTopics() {
        int rv = NONE;
        // default
        if (getResource().hasProperty(VBSVocabulary.incTopicsP)) {
            String incTopics = getResource().getPropertyValue(VBSVocabulary.incTopicsP).toString();
            if (incTopics.equals("ALL")) {
                rv = ALL;
            } else if (incTopics.equals("DIRECT")) {
                rv = DIRECT;
            }
        }
        return rv;
    }


    /**
     *  Gets the intesectionKey attribute of the JenaFacetSelector object
     *
     *@return    The intesectionKey value
     */
    public Object getIntesectionKey() {
        if (intersectionKey == null) {
            Iterator constraints = JenaResource.listPropertyValues(getResource(), VBSVocabulary.constraintP, OntResource.class);
            while (constraints.hasNext()) {
                OntResource node = (OntResource) constraints.next();
                if (intersectionKey == null) {
                    intersectionKey = "";
                } else {
                    intersectionKey += ", ";
                }
                intersectionKey += node.isAnon() ? node.getId().toString() : node.getURI();
            }
        }
        return intersectionKey;
    }


    /**
     *  Gets the maxBookmarks attribute of the JenaFacetSelector object
     *
     *@return    The maxBookmarks value
     */
    public Map getMaxBookmarks() {
        Map rv = null;
        if (getResource().hasProperty(VBSVocabulary.maxBookmarksP)) {
            rv = new TreeMap();
            putLimits(getResource().listPropertyValues(VBSVocabulary.maxBookmarksP), rv);
        }
        return rv;
    }


    /**
     *  Gets the maxSubCategories attribute of the JenaFacetSelector object
     *
     *@return    The maxSubCategories value
     */
    public Map getMaxSubCategories() {
        Map rv = null;
        if (getResource().hasProperty(VBSVocabulary.maxSubCategoriesP)) {
            rv = new TreeMap();
            putLimits(getResource().listPropertyValues(VBSVocabulary.maxSubCategoriesP), rv);
        }
        return rv;
    }


    /**
     *  Gets the skipBookmarks attribute of the JenaFacetSelector object
     *
     *@return    The skipBookmarks value
     */
    public Map getSkipBookmarks() {
        Map rv = null;
        if (getResource().hasProperty(VBSVocabulary.skipBookmarksP)) {
            rv = new TreeMap();
            putLimits(getResource().listPropertyValues(VBSVocabulary.skipBookmarksP), rv);
        }
        return rv;
    }


    /**
     *  Gets the skipSubCategories attribute of the JenaFacetSelector object
     *
     *@return    The skipSubCategories value
     */
    public Map getSkipSubCategories() {
        Map rv = null;
        if (getResource().hasProperty(VBSVocabulary.skipSubCategoriesP)) {
            rv = new TreeMap();
            putLimits(getResource().listPropertyValues(VBSVocabulary.skipSubCategoriesP), rv);
        }
        return rv;
    }


    /**
     *  Description of the Method
     *
     *@param  iter      Description of the Parameter
     *@param  limitMap  Description of the Parameter
     */
    private void putLimits(NodeIterator iter, Map limitMap) {
        while (iter.hasNext()) {
            OntResource limit = (OntResource) iter.nextNode().as(OntResource.class);
            String catid = "";
            String limitValue = "0";
            if (limit.hasProperty(VBSVocabulary.onCategoryP)) {
                catid = limit.getPropertyValue(VBSVocabulary.onCategoryP).toString();
            }
            if (limit.hasProperty(VBSVocabulary.limitP)) {
                limitValue = limit.getPropertyValue(VBSVocabulary.limitP).toString();
            }
            limitMap.put(catid, new Integer(limitValue));
        }
    }
}

