/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package vbsearch.impl.jena;

import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.shared.PrefixMapping;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;
import java.util.Iterator;
import seco.util.iterator.*;
import vbsearch.*;

/**
 *  Description of the Class
 *
 *@author     Samppa
 *@created    September 23, 2003
 */
public class JenaInputLoader implements VBSInputLoader {
    /**
     *  Description of the Field
     */
    String document;
    /**
     *  Description of the Field
     */
    OntModel model;


    /**
     *  Constructor for the JenaInputLoader object
     *
     *@param  model     Description of the Parameter
     *@param  document  Description of the Parameter
     */
    public JenaInputLoader(OntModel model, String document) {
        this.model = model;
        this.model.addSubModel(VBSVocabulary.getModel());
        this.document = document;
    }


    /**
     *  Gets the bookmarks attribute of the JenaInputLoader object
     *
     *@return    The bookmarks value
     */
    public Iterator getBookmarks() {
        return new BasicIterator();
    }


    /**
     *  Gets the categoryBase attribute of the JenaInputLoader object
     *
     *@return    The categoryBase value
     */
    public VBSResource getCategoryBase() {
        VBSResource rv = null;
        ExtendedIterator iter = ((OntClass) VBSVocabulary.categoryBaseC.inModel(model).as(OntClass.class))
                .listInstances();
        if (iter.hasNext()) {
            rv = new JenaResource((OntResource) iter.next());
        }
        iter.close();
        return rv;
    }


    /**
     *  Gets the prefixMapping attribute of the JenaInputLoader object
     *
     *@return    The prefixMapping value
     */
    public PrefixMapping getPrefixMapping() {
        return this.model;
    }


    /**
     *  Gets the rootCategories attribute of the JenaInputLoader object
     *
     *@return    The rootCategories value
     */
    public Iterator getRootCategories() {
        return new BasicIterator(JenaResource.listPropertyValues((Resource) VBSVocabulary.rootCategoryR.inModel(this.model),
                VBSVocabulary.subCategoriesP,
                OntResource.class),
            new IterationMap() {
                public Object map(Object o) {
                    return new JenaCategory(null,
                            (OntResource) o, document);
                }
            });
    }
}

