/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package vbsearch.impl.jena;

import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.util.iterator.*;
import java.util.*;
import seco.util.iterator.*;
import vbsearch.*;
import vbsearch.impl.*;

/**
 *  Description of the Class
 *
 *@author     Samppa
 *@created    September 11, 2003
 */
public class JenaResource implements VBSResource {

    /**
     *  Description of the Field
     */
    protected final static HashSet FILTERED_PROPERTIES = new HashSet();
    /**
     *  Description of the Field
     */
    protected String document;

    /**
     *  Description of the Field
     */
    private String id;

    /**
     *  Description of the Field
     */
    protected boolean isAnon;
    /**
     *  Description of the Field
     */
    protected Model model;
    /**
     *  Description of the Field
     */
    protected OntResource resource;

    static {
        FILTERED_PROPERTIES.add(VBSVocabulary.subCategoriesP.getURI());
        FILTERED_PROPERTIES.add(VBSVocabulary.topicOfP.getURI());
        FILTERED_PROPERTIES.add(VBSVocabulary.hasTopicP.getURI());
        FILTERED_PROPERTIES.add(VBSVocabulary.catidP.getURI());
        //FILTERED_PROPERTIES.add(VBSVocabulary..getURI());
    }


    /**
     *  Constructor for the JenaResource object
     *
     *@param  resource  Description of the Parameter
     */
    public JenaResource(OntResource resource) {
        this(resource, null);
    }


    /**
     *  Constructor for the JenaResource object
     *
     *@param  resource  Description of the Parameter
     *@param  document  Description of the Parameter
     */
    public JenaResource(OntResource resource, String document) {
        this.resource = resource;
        this.model = this.resource.getModel();
        this.isAnon = this.resource.isAnon();
        this.document = document;
        if (this.isAnon) {
            this.id = this.resource.getId().toString();
        } else {
            this.id = this.resource.getURI();
        }
    }


    /**
     *  Description of the Method
     *
     *@param  o  Description of the Parameter
     *@return    Description of the Return Value
     */
    public int compareTo(Object o) {
        VBSResource another = (VBSResource) o;
        String thiskey = this.id;
        String otherkey = another.isAnon() ? another.getID() : another.getURI();
        return thiskey.compareTo(otherkey);
    }


    /**
     *  Description of the Method
     *
     *@param  o  Description of the Parameter
     *@return    Description of the Return Value
     */
    public boolean equals(Object o) {
        return compareTo(o) == 0;
    }


    /**
     *  Gets the document attribute of the JenaResource object
     *
     *@return    The document value
     */
    public String getDocument() {
        return this.document;
    }


    /**
     *  Gets the iD attribute of the JenaResource object
     *
     *@return    The iD value
     */
    public String getID() {
        if (isAnon()) {
            return this.id;
        } else {
            return null;
        }
    }


    /**
     *  Gets the label attribute of the JenaResource object
     *
     *@param  lang  Description of the Parameter
     *@return       The label value
     */
    public String getLabel(String lang) {
        return this.resource.getLabel(lang);
    }


    /**
     *  Gets the localName attribute of the JenaResource object
     *
     *@return    The localName value
     */
    public String getLocalName() {
        return this.resource.getLocalName();
    }


    /**
     *  Gets the nameSpace attribute of the JenaResource object
     *
     *@return    The nameSpace value
     */
    public String getNamespace() {
        return this.resource.getNameSpace();
    }


    /**
     *  Gets the resource attribute of the JenaResource object
     *
     *@return    The resource value
     */
    protected OntResource getResource() {
        return this.resource;
    }


    /**
     *  Gets the uRI attribute of the BookmarkInfo object
     *
     *@return    The uRI value
     */
    public String getURI() {
        if (isAnon()) {
            return null;
        } else {
            return this.id;
        }
    }


    /**
     *  Gets the anon attribute of the JenaResource object
     *
     *@return    The anon value
     */
    public boolean isAnon() {
        return isAnon;
    }


    /**
     *  Gets the vBSResources attribute of the JenaResource class
     *
     *@param  subj  Description of the Parameter
     *@param  pred  Description of the Parameter
     *@param  TYPE  Description of the Parameter
     *@return       The vBSResources value
     */
    public static BasicIterator listPropertyValues(Resource subj, Property pred, final Class TYPE) {
        if (subj == null || pred == null) {
            return null;
        }
        BasicIterator rv = null;
        StmtIterator stmtiter = subj.listProperties(pred);
        while (stmtiter.hasNext()) {
            RDFNode value = stmtiter.nextStatement().getObject();
            if (value.canAs(RDFList.class)) {
                // rdf:parseC="Collection"?
                if (rv == null) {
                    rv = new BasicIterator(((RDFList) value.as(RDFList.class)).iterator());
                } else {
                    rv = rv.andThen(((RDFList) value.as(RDFList.class)).iterator());
                }
                /*
                 *  / Tämä ei jostain syystä toimi --> Ei käytetä Containereita!
                 *  } else if (value.canAs(Alt.class)) {
                 *  / rdf:type= Alt | Seq | Bag
                 *  if (rv == null) {
                 *  rv = new BasicIterator(((Alt) value.as(Alt.class)).iterator());
                 *  } else {
                 *  rv = rv.andThen(((Alt) value.as(Alt.class)).iterator());
                 *  }
                 *  } else if (value.canAs(Bag.class)) {
                 *  / rdf:type= Alt | Seq | Bag
                 *  if (rv == null) {
                 *  rv = new BasicIterator(((Bag) value.as(Bag.class)).iterator());
                 *  } else {
                 *  rv = rv.andThen(((Bag) value.as(Bag.class)).iterator());
                 *  }
                 *  } else if (value.canAs(Seq.class)) {
                 *  / rdf:type= Alt | Seq | Bag
                 *  if (rv == null) {
                 *  rv = new BasicIterator(((Seq) value.as(Seq.class)).iterator());
                 *  } else {
                 *  rv = rv.andThen(((Seq) value.as(Seq.class)).iterator());
                 *  }
                 *  }
                 */
            } else if (value.canAs(TYPE)) {
                // Requested TYPE?
                // if (DEBUG) {
                // System.out.println("get OntResources: value " + value + " is an OntResource.");
                // }
                if (rv == null) {
                    rv = new BasicIterator(value);
                } else {
                    rv = rv.andThen(new BasicIterator(value));
                }
            }
        }
        stmtiter.close();
        if (rv != null) {
            rv = new BasicIterator(rv,
//					new UniqueFilter(),
                new IterationMap() {
                    public Object map(Object o) {
                        return ((RDFNode) o).as(TYPE);
                    }
                });
        } else {
            rv = new BasicIterator();
        }
        return rv;
    }


    /**
     *  Description of the Method
     *
     *@param  property  Description of the Parameter
     *@return           Description of the Return Value
     */
    public Iterator listStatements(String property) {
        final StmtIterator statements;
        if (property == null) {
            statements = this.resource.listProperties();
        } else {
            statements = this.resource.listProperties(this.model.getProperty(property));
        }
        return new BasicIterator(statements,
            new IterationFilter() {
                /**
                 *  Filter off unwanted statements.
                 *
                 *@param  o
                 *@return
                 */
                public boolean accept(Object o) {
                    boolean rv = false;
                    Statement stmt = (Statement) o;
                    Property predicate = stmt.getPredicate();
                    if (predicate.getURI().equals(VBSConf.RDF_NS + "type")) {
                        String type = stmt.getResource().getURI();
                        if (type == null || !type.startsWith(VBSConf.OGT_NS) && !type.startsWith(VBSConf.BM_NS)) {
                            rv = true;
                        }
                    } else if (!FILTERED_PROPERTIES.contains(predicate.getURI())) {
                        rv = true;
                    }
                    return rv;
                }
            },
            new IterationMap() {
                public Object map(Object o) {
                    return new JenaStatement((Statement) o);
                }
            });
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public String toString() {
        if (isAnon()) {
            return getID();
        } else {
            return getURI();
        }
    }
}


