/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package vbsearch.impl.jena;

import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.util.iterator.*;
import java.io.*;
import java.util.*;
import seco.util.iterator.*;
import seco.io.*;
import vbsearch.*;
import vbsearch.impl.*;

/**
 *  Description of the Class
 *
 *@author     Samppa
 *@created    28. elokuuta 2003
 */
public class JenaVBSQuery implements VBSQuery {
    /**
     *  Description of the Field
     */
    private OntModel query;


    /**
     *  Constructor for the VBSQuery object
     *
     *@param  queryStr  Description of the Parameter
     */
    public JenaVBSQuery(String queryStr) {
        this(new StringInputStream(queryStr));
    }

    /**
     *  Constructor for the VBSQuery object
     *
     *@param  in  Description of the Parameter
     */
    public JenaVBSQuery(InputStream in) {
        this((OntModel) ModelFactory.createOntologyModel(VBSConf.ONT_MODEL_SPEC, null)
                .read(in, VBSConf.BASE_URI));
    }

    /**
     *  Constructor for the VBSQuery object
     *
     *@param  query  Description of the Parameter
     */
    public JenaVBSQuery(OntModel query) {
        this.query = query;
        this.query.addSubModel(VBSVocabulary.getModel());
    }


    /**
     *  Gets the facetSelectors attribute of the VBSQuery object
     *
     *@return    The facetSelectors value
     */
    public Iterator getFacetSelectors() {
        return new BasicIterator(((OntClass) VBSVocabulary.facetSelectorC.inModel(this.query).as(OntClass.class))
                .listInstances(),
            new IterationMap() {
                public Object map(Object o) {
                    return new JenaFacetSelector((OntResource) o);
                }
            });
    }
}

