/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package vbsearch.impl.mem;

import java.io.Serializable;
import java.net.*;
import java.util.*;
import seco.util.iterator.*;
import vbsearch.*;
import vbsearch.impl.*;
import vbsearch.util.*;

/**
 *  Description of the Class
 *
 *@author     Samppa
 *@created    26. elokuuta 2003
 */
public class BookmarkImpl extends VBSResourceImpl implements Bookmark, Serializable {
    /**
     *  Description of the Field
     */
    private SortedSet categories = new TreeSet();


    /**
     *  Constructor for the BookmarkImpl object
     *
     *@param  bminfo                    Description of the Parameter
     *@exception  URISyntaxException    Description of the Exception
     *@exception  NullPointerException  Description of the Exception
     */
    protected BookmarkImpl(Bookmark bminfo) throws URISyntaxException, NullPointerException {
        super(bminfo);
    }


    /**
     *  Adds a feature to the Category attribute of the Bookmark object
     *
     *@param  category  The feature to be added to the Category attribute
     *@param  catid     The feature to be added to the Category attribute
     */
    protected void addCategory(Category category, PrefixLabel catid) {
        this.categories.add(new CategoryLink(category, catid));
    }


    /**
     *  Gets the categories attribute of the Bookmark object
     *
     *@return    The categories value
     */
    public Iterator getCategories() {
        return new BasicIterator(this.categories.iterator(),
            new IterationMap() {
                public Object map(Object o) {
                    return ((CategoryLink) o).category;
                }
            });
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    protected boolean hasCategories() {
        return this.categories.size() > 0;
    }


    /**
     *  Description of the Method
     *
     *@param  category  Description of the Parameter
     *@return           Description of the Return Value
     */
    protected boolean hasCategory(Category category) {
        return this.categories.contains(new CategoryLink(category, null));
    }


    /**
     *  Description of the Method
     *
     *@param  category                      Description of the Parameter
     *@return                               Description of the Return Value
     *@exception  UnknownResourceException  Description of the Exception
     */
    protected PrefixLabel removeCategory(Category category) throws UnknownResourceException {
        CategoryLink catlink = null;
        CategoryLink key = new CategoryLink(category, null);
        if (this.categories.contains(key)) {
            catlink = (CategoryLink) this.categories.tailSet(key).first();
            this.categories.remove(catlink);
        } else {
            throw new UnknownResourceException(category);
        }
        return catlink.catid;
    }


    /**
     *  Description of the Class
     *
     *@author     Samppa
     *@created    September 19, 2003
     */
    private class CategoryLink implements Comparable, Serializable {
        /**
         *  Description of the Field
         */
        public Category category;
        /**
         *  Description of the Field
         */
        public PrefixLabel catid;


        /**
         *  Constructor for the CategoryLink object
         *
         *@param  category  Description of the Parameter
         *@param  catid     Description of the Parameter
         */
        private CategoryLink(Category category, PrefixLabel catid) {
            this.category = category;
            this.catid = catid;
        }


        /**
         *  Description of the Method
         *
         *@param  another  Description of the Parameter
         *@return          Description of the Return Value
         */
        public int compareTo(Object another) {
            return this.category.compareTo(((CategoryLink) another).category);
        }
    }
}

