/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package vbsearch.impl.mem;

import java.io.Serializable;
import java.net.*;
import java.util.*;
import vbsearch.*;
import vbsearch.impl.*;
import vbsearch.util.*;

/**
 *  Description of the Class
 *
 *@author     Samppa Saarela
 *@created    26. elokuuta 2003
 */
public class CategoryImpl extends VBSResourceImpl implements Category, Serializable {
    /**
     *  Description of the Field
     */
    private SortedSet bookmarks = new TreeSet();
    /**
     *  Description of the Field
     */
    private PrefixLabel catid;
    /**
     *  Description of the Field
     */
    private LinkedList children = new LinkedList();
    /**
     *  Description of the Field
     */
    private VBSIndexImpl index;
    /**
     *  Description of the Field
     */
    private Category parent;


    /**
     *  Constructor for the CategoryImpl object
     *
     *@param  parent                    Description of the Parameter
     *@param  catinfo                   Description of the Parameter
     *@param  index                     Description of the Parameter
     *@exception  URISyntaxException    Description of the Exception
     *@exception  NullPointerException  Description of the Exception
     */
    protected CategoryImpl(CategoryImpl parent, Category catinfo, VBSIndexImpl index) throws URISyntaxException, NullPointerException {
        super(catinfo);
        this.parent = parent;
        this.index = index;
    }


    /**
     *  Adds a feature to the Bookmark attribute of the Category object
     *
     *@param  bookmark  The feature to be added to the Bookmark attribute
     */
    protected void addBookmark(BookmarkImpl bookmark) {
        this.bookmarks.add(bookmark);
    }


    /**
     *  Adds a feature to the SubCategory attribute of the Category object
     *
     *@param  child  The feature to be added to the SubCategory attribute
     */
    protected void addSubCategory(CategoryImpl child) {
        if (!this.children.contains(child))
            this.children.addLast(child);
    }


    /**
     *  Gets the bookmarks attribute of the Category object
     *
     *@param  direct  Description of the Parameter
     *@return         The bookmarks value
     */
    public Iterator getBookmarks(boolean direct) {
        if (direct) {
            return this.bookmarks.iterator();
        } else {
            return this.index.getBookmarks(this).iterator();
        }
    }


    /**
     *  Gets the categoryID attribute of the Category object
     *
     *@return    The categoryID value
     */
    public PrefixLabel getCategoryID() {
        return this.catid;
    }


    /**
     *  Gets the parent attribute of the Category object
     *
     *@return    The parent value
     */
    public Category getParent() {
        return this.parent;
    }


    /**
     *  Gets the children attribute of the Category object
     *
     *@param  direct  Description of the Parameter
     *@return         The children value
     */
    public Iterator getSubCategories(boolean direct) {
        if (direct) {
            return this.children.iterator();
        } else {
            return this.index.getSubCategories(this).iterator();
        }
    }


    /**
     *  Description of the Method
     *
     *@param  bookmark  Description of the Parameter
     *@return           Description of the Return Value
     */
    protected boolean hasBookmark(Bookmark bookmark) {
        return this.bookmarks.contains(bookmark);
    }


    /**
     *  Description of the Method
     *
     *@param  child  Description of the Parameter
     *@return        Description of the Return Value
     */
    protected boolean hasSubCategory(Category child) {
        return this.children.contains(child);
    }


    /**
     *  Description of the Method
     *
     *@param  bookmark  Description of the Parameter
     */
    protected void removeBookmark(BookmarkImpl bookmark) {
        this.bookmarks.remove(bookmark);
    }


    /**
     *  Description of the Method
     *
     *@param  category  Description of the Parameter
     */
    protected void removeSubCategory(CategoryImpl category) {
        this.children.remove(category);
    }


    /**
     *  Sets the categoryID attribute of the Category object
     *
     *@param  catid  The new categoryID value
     */
    protected void setCategoryID(PrefixLabel catid) {
        this.catid = catid;
    }
}

