/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package vbsearch.query;

import com.hp.hpl.jena.ontology.*;
import java.util.*;
import vbsearch.*;

/**
 *  Description of the Class
 *
 *@author     Samppa
 *@created    September 12, 2003
 */
public interface FacetSelector extends CategoryConstraint, VBSResource {
    /**
     *  Description of the Field
     */
    public final static int ALL = -1;

    /**
     *  Description of the Field
     */
    public final static int DIRECT = 1;

    /**
     *  Description of the Field
     */
    public final static int NONE = 0;

    /**
     *  Description of the Field
     */
    public final static int PRE_QUERY = 1;

    /**
     *  Description of the Field
     */
    public final static int POST_QUERY = 2;


    /**
     *  Gets the constraint attribute of the FacetSelector object
     *
     *@return    The constraint value
     */
    public Iterator getConstraints();


    /**
     *  Gets the incBookmarks attribute of the FacetSelector object
     *
     *@return    The incBookmarks value
     */
    public boolean getIncBookmarks();


    /**
     *  Gets the incPath attribute of the FacetSelector object
     *
     *@return    The incPath value
     */
    public boolean getIncPath();


    /**
     *@return
     */
    public Object getIntesectionKey();


    /**
     *  Gets the incRemainder attribute of the FacetSelector object
     *
     *@return    The incRemainder value
     */
    public boolean getIncRemainder();


    /**
     *  Gets the incDirectBookmarkHits attribute of the FacetSelector object
     *
     *@return    The incDirectBookmarkHits value
     */
    public boolean getIncDirectBookmarkHits();


    /**
     *@return
     */
    public Map getMaxSubCategories();


    /**
     *@return
     */
    public boolean getIncSubCategoryHits();


    /**
     *  Gets the incPosition attribute of the FacetSelector object
     *
     *@return    The incPosition value
     */
    public boolean getIncPosition();


    /**
     *@return
     */
    public boolean getIncPathSubCategoryHits();


    /**
     *@return
     */
    public Map getMaxBookmarks();


    /**
     *  Gets the incCategories attribute of the FacetSelector object
     *
     *@return    The incCategories value
     */
    public boolean getIncCategories();


    /**
     *  Gets the incHits attribute of the FacetSelector object
     *
     *@return    The incHits value
     */
    public boolean getIncBookmarkHits();


    /**
     *  Gets the incHits attribute of the FacetSelector object
     *
     *@return    The incHits value
     */
    public boolean getIncPathBookmarkHits();


    /**
     *@return
     */
    public Map getSkipBookmarks();


    /**
     *@return
     */
    public Map getSkipSubCategories();


    /**
     *  Gets the incSubCategories attribute of the FacetSelector object
     *
     *@return    The incSubCategories value
     */
    public int getIncSubCategories();


    /**
     *  Gets the incProjectionOf attribute of the FacetSelector object
     *
     *@return    The incProjectionOf value
     */
    public java.util.Iterator getIncBookmarkProperty();


    /**
     *  Gets the incCategoryProperty attribute of the VBSSelector object
     *
     *@return    The incCategoryProperty value
     */
    public java.util.Iterator getIncCategoryProperty();


    /**
     *  Gets the acceptLanguage attribute of the FacetSelector object
     *
     *@return    The acceptLanguage value
     */
    public String getAcceptLanguage();


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public int getFilter();


    /**
     *  Gets the incTopics attribute of the FacetSelector object
     *
     *@return    The incTopics value
     */
    public int getIncTopics();
}

