/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package vbsearch.util;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 *  Description of the Class
 *
 *@author     Samppa
 *@created    14. elokuuta 2003
 */
public class PrefixLabel implements Comparable, Serializable {
    /**
     *  Description of the Field
     */
    protected final static String CHARACTER_ENCODING = "UTF-8";
    /**
     *  Description of the Field
     */
    protected final static char FIRST_ID = Character.MIN_VALUE;
    /**
     *  Description of the Field
     */
    protected String label;
    /**
     *  Description of the Field
     */
    protected PrefixLabel lastChild;
    /**
     *  Description of the Field
     */

//	{ // Uncomment this for debugging

//		child = 'A';
//	}
    /**
     *  Constructor for the PrefixLabel object
     *
     *@param  label    Description of the Parameter
     *@param  encoded  Description of the Parameter
     */
//	public PrefixLabel(String label) {
//		this(label, true);
//	}


    /**
     *  Constructor for the PrefixLabel object
     *
     *@param  label    Description of the Parameter
     *@param  encoded  Description of the Parameter
     */
    public PrefixLabel(String label, boolean encoded) {
        if (encoded) {
            try {
                this.label = URLDecoder.decode(label, CHARACTER_ENCODING);
            } catch (UnsupportedEncodingException uee) {
                uee.printStackTrace();
                System.exit(0);
            }
        } else {
            this.label = label;
        }
    }


    /**
     *  Constructor for the PrefixLabel object
     *
     *@param  id  Description of the Parameter
     */
    protected PrefixLabel(char id) {
        this("" + id, false);
    }


    /**
     *  Constructor for the PrefixLabel object
     *
     *@param  id      Description of the Parameter
     *@param  parent  Description of the Parameter
     */
    protected PrefixLabel(PrefixLabel parent, char id) {
        this("" + parent + id, false);
    }


    /**
     *  Description of the Method
     *
     *@param  another  Description of the Parameter
     *@return          Description of the Return Value
     */
    public int compareTo(Object another) {
        return this.label.compareTo(((PrefixLabel) another).label);
    }


    /**
     *  Description of the Method
     *
     *@param  another  Description of the Parameter
     *@return          Description of the Return Value
     */
    public boolean equals(Object another) {
        return this.label.equals(((PrefixLabel) another).label);
    }


    /**
     *  Gets the iD attribute of the PrefixLabel object
     *
     *@return    The iD value
     */
    protected int getID() {
        return label.charAt(label.length() - 1);
    }


    /**
     *  Gets the next attribute of the PrefixLabel object
     *
     *@return    The next value
     */
    public PrefixLabel getNext() {
        PrefixLabel next = null;
        if (getNextID() >= Character.MIN_VALUE && getNextID() <= Character.MAX_VALUE) {
            next = new PrefixLabel(getNextLabel(), false);
        } else {
            throw new LabelIDOutOfBoundsException();
        }
        return next;
    }


    /**
     *  Gets the nextID attribute of the PrefixLabel object
     *
     *@return    The nextID value
     */
    protected int getNextID() {
        return getID() + 1;
    }


    /**
     *  Gets the nextLabel attribute of the PrefixLabel object
     *
     *@return    The nextLabel value
     */
    protected String getNextLabel() {
        return "" + getParentLabel() + (char) getNextID();
    }


    /**
     *  Gets the nextSub attribute of the PrefixLabel object
     *
     *@return    The nextSub value
     */
    public PrefixLabel getNextSub() {
        if (this.lastChild == null) {
            this.lastChild = new PrefixLabel(this, FIRST_ID);
        } else {
            this.lastChild = this.lastChild.getNext();
        }
        return this.lastChild;
    }


    /**
     *  Gets the parent attribute of the PrefixLabel object
     *
     *@return    The parent value
     */
    public PrefixLabel getParent() {
        return new PrefixLabel(getParentLabel(), false);
    }


    /**
     *  Gets the parentLabel attribute of the PrefixLabel object
     *
     *@return    The parentLabel value
     */
    protected String getParentLabel() {
        return label.substring(0, label.length() - 1);
    }


    /**
     *  Gets the prevID attribute of the PrefixLabel object
     *
     *@return    The prevID value
     */
    protected int getPrevID() {
        return getID() - 1;
    }


    /**
     *  Gets the prevLabel attribute of the PrefixLabel object
     *
     *@return    The prevLabel value
     */
    protected String getPrevLabel() {
        return "" + getParentLabel() + (char) getPrevID();
    }


    /**
     *  Gets the previous attribute of the PrefixLabel object
     *
     *@return    The previous value
     */
    public PrefixLabel getPrevious() {
        PrefixLabel prev = null;
        if (getPrevID() >= Character.MIN_VALUE && getPrevID() <= Character.MAX_VALUE) {
            prev = new PrefixLabel(getPrevLabel(), false);
        } else {
            throw new LabelIDOutOfBoundsException();
        }
        return prev;
    }


    /**
     *  Gets the root attribute of the PrefixLabel class
     *
     *@return    The root value
     */
    public static PrefixLabel getRoot() {
        return new PrefixLabel(FIRST_ID);
    }


    /**
     *  Gets the string attribute of the PrefixLabel object
     *
     *@return    The string value
     */
    public String getString() {
        return this.label;
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public int hashCode() {
        return this.label.hashCode();
    }


    /**
     *  The main program for the PrefixLabel class
     *
     *@param  args  The command line arguments
     */
    public static void main(String[] args) {
        PrefixLabel root = PrefixLabel.getRoot();
        TreeMap index = new TreeMap();
        index.put(root, "0");

        for (int i = 0; i < 128; i++) {
            PrefixLabel parent = root.getNextSub();
            index.put(parent, "" + root + "." + i +
                    "  {parent=" + parent.getParent() + ", prev=" + parent.getPrevious() + ", next=" + parent.getNext() + "}");
            for (int j = 0; j < 5; j++) {
                PrefixLabel child = parent.getNextSub();
                index.put(child, "" + root + "." + i + "." + j +
                        "  {parent=" + child.getParent() + ", prev=" + child.getPrevious() + ", next=" + child.getNext() + "}");
            }
        }
        for (Iterator iter = index.entrySet().iterator(); iter.hasNext(); ) {
            System.out.println(iter.next().toString());
        }
        System.out.println("" + root.getPrevious());
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public String toString() {
        return toString(false);
    }


    /**
     *  Description of the Method
     *
     *@param  encode  Description of the Parameter
     *@return         Description of the Return Value
     */
    public String toString(boolean encode) {
        String rv = null;
        if (encode) {
            try {
                rv = URLEncoder.encode(this.label, CHARACTER_ENCODING);
            } catch (UnsupportedEncodingException uee) {
                uee.printStackTrace();
                System.exit(0);
            }
        } else {
            rv = this.label;
        }
        return rv;
    }
}

